"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var logger_1 = require("../logger");
var Q = require('q');
var _ = require('lodash');
var access_control_with_organizations_authorization_1 = require("./access-control-with-organizations.authorization");
var controller_utils_1 = require("../utils/controller.utils");
var entity_utils_1 = require("../utils/entity.utils");
var base_query_builder_1 = require("../utils/base.query-builder");
var promise_grants_utils_1 = require("../utils/promise-grants.utils");
var functions_utils_1 = require("../utils/functions.utils");
var createError = require('http-errors');
var AccessControlBaseController = /** @class */ (function () {
    function AccessControlBaseController(model, grants) {
        this.model = model;
        var _resource = model.collection.collectionName;
        this.promisedAc = access_control_with_organizations_authorization_1.crudAccessControlWithOrgRolesFactory(_resource, grants);
        logger_1.default.debug("inited access control for " + _resource);
    }
    AccessControlBaseController.prototype.create = function (req, res) {
        var self = this;
        return controller_utils_1.baseHandle(req, res, self.promisedAc, 'create', function (grant, user) {
            return Q.when()
                .then(self.entityFromBody(req))
                .then(self.setOwner(grant, user))
                .then(entity_utils_1.createEntity(self.model));
        });
    };
    AccessControlBaseController.prototype.find = function (req, res, exQueryBuilder) {
        var self = this;
        return controller_utils_1.baseHandle(req, res, self.promisedAc, 'read', function (grant, user) {
            return Q.when()
                .then(self.restrictedQueryBuilderFactory(grant, user, exQueryBuilder))
                .then(base_query_builder_1.createQueryExecutor(self.model))
                .then(base_query_builder_1.execFindAndCount(req.query, res));
        });
    };
    AccessControlBaseController.prototype.findById = function (req, res, exQueryBuilder) {
        var self = this;
        return controller_utils_1.baseHandle(req, res, self.promisedAc, 'read', function (grant, user) {
            return Q.when()
                .then(self.restrictedQueryBuilderFactory(grant, user, exQueryBuilder))
                .then(base_query_builder_1.execFindByIdWithQueryBuilder(self.model, req.params.id))
                .then(controller_utils_1.handleEntityNotFound(res));
        });
    };
    AccessControlBaseController.prototype.restrictedQueryBuilderFactory = function (grant, user, exQueryBuilder) {
        return function () {
            var restrictedQueryBuilder = function (query) {
                var organizationCode = user.organization && user.organization.ref ? user.organization.ref.code : undefined;
                return functions_utils_1.now(query)
                    .then(promise_grants_utils_1.ifGrantedForOwn(grant, base_query_builder_1.restrictByOwner(grant.ownerTypes, user._id, organizationCode)))
                    .then(promise_grants_utils_1.ifDefined(exQueryBuilder))
                    .value();
            };
            return restrictedQueryBuilder;
        };
    };
    AccessControlBaseController.prototype.update = function (req, res) {
        var self = this;
        return controller_utils_1.baseHandle(req, res, self.promisedAc, 'update', function (grant, user) {
            return Q.when()
                .then(self.restrictedQueryBuilderFactory(grant, user))
                .then(base_query_builder_1.execFindByIdWithQueryBuilder(self.model, req.params.id))
                .then(controller_utils_1.handleEntityNotFound(res))
                .then(entity_utils_1.applyUpdate(req.body));
        });
    };
    AccessControlBaseController.prototype.patch = function (req, res) {
        var self = this;
        return controller_utils_1.baseHandle(req, res, self.promisedAc, 'update', function (grant, user) {
            return Q.when()
                .then(self.restrictedQueryBuilderFactory(grant, user))
                .then(base_query_builder_1.execFindByIdWithQueryBuilder(self.model, req.params.id))
                .then(controller_utils_1.handleEntityNotFound(res))
                .then(entity_utils_1.applyPatch(req.body));
        });
    };
    AccessControlBaseController.prototype.remove = function (req, res) {
        var self = this;
        return controller_utils_1.baseHandle(req, res, self.promisedAc, 'delete', function (grant, user) {
            return Q.when()
                .then(self.restrictedQueryBuilderFactory(grant, user))
                .then(base_query_builder_1.execFindByIdWithQueryBuilder(self.model, req.params.id))
                .then(controller_utils_1.handleEntityNotFound(res))
                .then(entity_utils_1.removeEntity())
                .then(controller_utils_1.successMessageResult());
        });
    };
    AccessControlBaseController.prototype.setOwner = function (grant, user) {
        return function (entity) {
            logger_1.default.trace('will set owner');
            logger_1.default.trace(grant);
            return functions_utils_1.now(entity)
                .then(promise_grants_utils_1.ifGrantedForUser(grant, entity_utils_1.setUserOwner(user)))
                .then(promise_grants_utils_1.ifGrantedForOrganization(grant, entity_utils_1.setOrganizationOwner(user)))
                .value();
        };
    };
    AccessControlBaseController.prototype.findBydId = function (id, user, grant) {
        var self = this;
        return function () {
            return Q.when(self.model.findById(id));
        };
    };
    AccessControlBaseController.prototype.entityFromBody = function (req) {
        return function () {
            var entity = req.body;
            return entity;
        };
    };
    return AccessControlBaseController;
}());
exports.AccessControlBaseController = AccessControlBaseController;
